------------------------------------------------------------------------------
--  ------------------         M a R T E   O S         -------------------  --
------------------------------------------------------------------------------
--                                                             V1.7     071213
--
--               'K e r n e l . I n i t i a l i z a t i o n'
--
--                                  Body
--
--
--  File 'kernel-initialization.adb'                                   By MAR.
--
--
--  Preforms the initialization of the kernel calling to the 'Initialize'
--  procedures of the others packages. It is the only package with begin-end
--  block.
--
--  ----------------------------------------------------------------------
--   Copyright (C) 2000-2007, Universidad de Cantabria, SPAIN
--
--   MaRTE OS web page: http://marte.unican.es
--   Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
--                      Michael Gonzalez Harbour      mgh@unican.es
--
--  MaRTE OS  is free software; you can  redistribute it and/or  modify it
--  under the terms of the GNU General Public License  as published by the
--  Free Software Foundation;  either  version 2, or (at  your option) any
--  later version.
--
--  MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
--  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
--  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
--  General Public License for more details.
--
--  You should have received  a  copy of  the  GNU General Public  License
--  distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
--  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
--  02111-1307, USA.
--
--  As a  special exception, if you  link this  unit  with other  files to
--  produce an   executable,   this unit  does  not  by  itself cause  the
--  resulting executable to be covered by the  GNU General Public License.
--  This exception does  not however invalidate  any other reasons why the
--  executable file might be covered by the GNU Public License.
--
------------------------------------------------------------------------------
with Hardware_Interface;
pragma Elaborate_All (Hardware_Interface);
with Kernel.Timers;
pragma Elaborate_All (Kernel.Timers);
with Kernel.Timed_Events_Queue;
pragma Elaborate_All (Kernel.Timed_Events_Queue);
with Kernel.Hardware_Interrupts;
pragma Elaborate_All (Kernel.Hardware_Interrupts);
with Kernel.Signals.Posix_Functions;
pragma Elaborate_All (Kernel.Signals.Posix_Functions);
with Kernel.Replenishment_Te;
pragma Elaborate_All (Kernel.Replenishment_Te);
with Kernel.Signals.Internals;
pragma Elaborate_All (Kernel.Signals.Internals);
with Kernel.Timer_Timed_Events_Pool;
pragma Elaborate_All (Kernel.Timer_Timed_Events_Pool);
with Kernel.Pool_TCBs;
pragma Elaborate_All (Kernel.Pool_TCBs);
with Kernel.Scheduler;
pragma Elaborate_All (Kernel.Scheduler);
with Kernel.Signals.Handler;
pragma Elaborate_All (Kernel.Signals.Handler);
with Kernel.Devices_Table;
pragma Elaborate_All (Kernel.Devices_Table);
with Kernel_Console;
pragma Elaborate_All (Kernel_Console);
with Kernel.Debug;
pragma Elaborate_All (Kernel.Debug);
with Kernel.Timed_Events_And_Timer;
pragma Elaborate_All (Kernel.Timed_Events_And_Timer);
with Kernel.Task_Sets.Operations;
pragma Elaborate_All (Kernel.Task_Sets.Operations);
with Configuration_Parameters;

package body Kernel.Initialization is

   package HWI renames Hardware_Interface;
   package CP  renames Configuration_Parameters;
   use type CP.Supported_Architectures;

   Initialized : Boolean := False;

   procedure Initialize is
   begin
      if Initialized then
         --  This procedure can be called from two places:
         --  System.OS_Interface.pthread_init and the begin-end block
         --  of this package.
         return;
      end if;
      Initialized := True;

      --  Initialize the hardware interface
      HWI.Initialize;
      pragma Assert (CP.MaRTE_Architecture'First /= CP.ARCH_X86 or else
                     not HWI.Are_Interrupts_Enabled);
      --  HWI.Are_Interrupts_Enabled return true in Linux and Linux_Lib because
      --  interrupts are "enabled" after every call to Leave_Kernel.

      Kernel.Timers.Initialize_Realtime_Clock_Offset;

      --  Initialize the kernel debug messages
      Kernel.Debug.Initialize;

      --  Initialize timed events a timer
      Kernel.Timed_Events_And_Timer.Initialize;

      --  Initialize kernel structures and pools
      Kernel.Task_Sets.Operations.Initialize;
      Kernel.Timed_Events_Queue.Initialize;
      Kernel.Hardware_Interrupts.Initialize;
      Kernel.Signals.Posix_Functions.Initialize;
      Kernel.Replenishment_Te.Initialize_Pool;
      Kernel.Signals.Internals.Initialize;
      Kernel.Timer_Timed_Events_Pool.Initialize_Pool;

      --  Initialize resources for tasks
      Kernel.Pool_TCBs.Initialize;

      --  Initialize scheduler structures, main and idle tasks and install the
      --  Timer interrupt handler
      Kernel.Scheduler.Initialize;

      --  Initialize Kernel_Nesting_Level
      Kernel.Kernel_Nesting_Level := 0;

      --  Initialize the signal handler task
      Kernel.Signals.Handler.Initialize_Signal_Handler_Task;

      --  Initialize drivers
      pragma Assert (CP.MaRTE_Architecture'First /= CP.ARCH_X86 or else
                     not HWI.Are_Interrupts_Enabled);
      Kernel.Devices_Table.Initialize_Devices;

      --  Inform the console kernel initialization has finished
      Kernel_Console.End_Of_Kernel_Initialization;

      --  Enable interrupts: at this point Kernel is set up so we are ready to
      --  enable interrupts and start task scheduling
      HWI.Interrupt_Controller_Hardware_Enabled;
      HWI.Interrupts_Enabled;

   end Initialize;

begin
   Initialize;
end Kernel.Initialization;
