------------------------------------------------------------------------------
--  ------------------         M a R T E   O S         -------------------  --
------------------------------------------------------------------------------
--                                                             V1.7     071213
--
--                      'T a s k s _ I n s p e c t o r'
--
--                                 Spec
--
--
--  File 'tasks_inspector.ads'                                        By MAR.
--
--
--  Operantions to send scheduling events to the host using a serial
--  port. The events sent using 'Send_Event' can be interpretated by the
--  "Tasks Inspector" tool to display the tasks activity graphically.
--
--  ----------------------------------------------------------------------
--   Copyright (C) 2000-2007, Universidad de Cantabria, SPAIN
--
--   MaRTE OS web page: http://marte.unican.es
--   Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
--                      Michael Gonzalez Harbour      mgh@unican.es
--
--  MaRTE OS  is free software; you can  redistribute it and/or  modify it
--  under the terms of the GNU General Public License  as published by the
--  Free Software Foundation;  either  version 2, or (at  your option) any
--  later version.
--
--  MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
--  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
--  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
--  General Public License for more details.
--
--  You should have received  a  copy of  the  GNU General Public  License
--  distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
--  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
--  02111-1307, USA.
--
--  As a  special exception, if you  link this  unit  with other  files to
--  produce an   executable,   this unit  does  not  by  itself cause  the
--  resulting executable to be covered by the  GNU General Public License.
--  This exception does  not however invalidate  any other reasons why the
--  executable file might be covered by the GNU Public License.
--
-------------------------------------------------------------------------------
with Hardware_Interface;
with Kernel;

package Tasks_Inspector is

   procedure Init (Num_Tasks_Mx     : in Integer;
                   Task_Priority_Mx : in Integer;
                   HWClock_Freq     : in Hardware_Interface.HWTime);
   --  Initialize the serial port to be used by the tasks inspector tool
   --  ('Debug_Messages.Task_Inspector_Serial_Port'). Called from the
   --  'begin-end' of 'Kernel.Scheduler.Debug'.
   pragma Inline (Init);

   procedure Send_Event (Timestamp : in Hardware_Interface.HWTime;
                         Operation : in String;
                         T1        : in Kernel.Task_Id;
                         T2        : in Kernel.Task_Id);

   procedure Send_Event (Timestamp : in Hardware_Interface.HWTime;
                         Operation : in String;
                         Id        : in Integer;
                         Prio      : in Kernel.Task_Priority);

   pragma Inline (Send_Event);
end Tasks_Inspector;

