------------------------------------------------------------------------------
--  ------------------         M a R T E   O S         -------------------  --
------------------------------------------------------------------------------
--                                                             V1.7     071213
--
--                        'T y p e _ T i m e s p e c'
--
--                                   Body
--
--
--
--  File 'type_timespec.adb'                                           By MAR.
--
--  This package is a part of the layer that implements the POSIX.C
--  funcionalty over MaRTE OS.
--
--  ----------------------------------------------------------------------
--   Copyright (C) 2000-2007, Universidad de Cantabria, SPAIN
--
--   MaRTE OS web page: http://marte.unican.es
--   Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
--                      Michael Gonzalez Harbour      mgh@unican.es
--
--  MaRTE OS  is free software; you can  redistribute it and/or  modify it
--  under the terms of the GNU General Public License  as published by the
--  Free Software Foundation;  either  version 2, or (at  your option) any
--  later version.
--
--  MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
--  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
--  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
--  General Public License for more details.
--
--  You should have received  a  copy of  the  GNU General Public  License
--  distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
--  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
--  02111-1307, USA.
--
--  As a  special exception, if you  link this  unit  with other  files to
--  produce an   executable,   this unit  does  not  by  itself cause  the
--  resulting executable to be covered by the  GNU General Public License.
--  This exception does  not however invalidate  any other reasons why the
--  executable file might be covered by the GNU Public License.
--
------------------------------------------------------------------------------
with Unchecked_Conversion;
with Hardware_Interface;
pragma Elaborate_All (Hardware_Interface);

package body Type_Timespec is

   package HWI renames Hardware_Interface; use HWI;

   HWT_HZ : constant HWI.HWTime := HWI.Get_HWClock_Frequency;

   type Ts is record
      Sec : Int;
      Nsec : Int;
   end record;
   function To_Ts is new Unchecked_Conversion (HWI.HWTime, Ts);
   function HWTime_To_Timespec (Th : in HWI.HWTime) return Timespec is
      S : HWI.HWTime;
   begin
      S := Th / HWT_HZ;
      return Timespec'(Tv_Sec  => To_Ts (S).Sec,
                       Tv_Nsec => To_Ts (((Th - S * HWT_HZ) * 1_000_000_000) /
                                      HWT_HZ).Sec);
   end HWTime_To_Timespec;
   pragma Inline (HWTime_To_Timespec);

   ------------------------
   -- Timespec_To_HWTime --
   ------------------------
   function Timespec_To_HWTime (TS : in Timespec) return HWI.HWTime is
   begin
      return HWI.HWTime (TS.Tv_Sec) * HWT_HZ
        + (HWI.HWTime (TS.Tv_Nsec) * HWT_HZ) / 1_000_000_000;
   end Timespec_To_HWTime;
   pragma Inline (Timespec_To_HWTime);

   -----------------
   -- To_Duration --
   -----------------
   function To_Duration (TS : Timespec) return Duration is
   begin
      return Duration (TS.Tv_Sec) + Duration (TS.Tv_Nsec) / 10#1#E9;
   end To_Duration;

   -----------------
   -- To_Timespec --
   -----------------
   function Dur_To_U64 is new Unchecked_Conversion (Duration, Unsigned_64);
   function To_Timespec (D : Duration) return Timespec is
   begin
      return Timespec'(Tv_Sec  => Int (Dur_To_U64 (D) / 1_000_000_000),
                       Tv_Nsec => Int (Dur_To_U64 (D) mod 1_000_000_000));
   end To_Timespec;

end Type_Timespec;
