
with Ada.Unchecked_Conversion;

with Kernel.Tasks_Operations.Attributes;
with Kernel.Tasks_Operations.Application_Scheduler;

package body System.OS_Interface.Application_Scheduling is

   package TO_APPSCHD renames Kernel.Tasks_Operations.Application_Scheduler;
   package TO_ATTR    renames Kernel.Tasks_Operations.Attributes;

   function To_MaRTE_Pthread_T is
     new Ada.Unchecked_Conversion (pthread_t, Pthread.Pthread_T);
   function To_MaRTE_Param_Ac is
      new Ada.Unchecked_Conversion (Scheduling_Parameters_Ac,
                                    Kernel.AppSched_Param_T_Ac);

   ----------------------------------------------------------------------------
   --  Set and Get Application Defined Sched Parameters  ----------------------
   ----------------------------------------------------------------------------

   -------------------------------------
   --  Pthread_Attr_Setappschedparam  --
   -------------------------------------
   function Pthread_Attr_Setappschedparam
     (Attr                : access pthread_attr_t;
      AppSched_Param      : in     Scheduling_Parameters_Ac;
      AppSched_Param_Size : in     Scheduling_Parameters_Size) return int is
   begin
      return TO_ATTR.Pthread_Attr_Setappschedparam
        (Attr, To_MaRTE_Param_Ac (AppSched_Param),
         Kernel.AppSched_Param_Size_T (AppSched_Param_Size));
   end Pthread_Attr_Setappschedparam;

   --------------------------------
   --  Pthread_Setappschedparam  --
   --------------------------------
   function Pthread_Setappschedparam
     (T                   : in     pthread_t;
      AppSched_Param      : access Scheduling_Parameters'Class;
      AppSched_Param_Size : in     Scheduling_Parameters_Size) return int is
   begin
      return TO_APPSCHD.Pthread_Setappschedparam
        (To_MaRTE_Pthread_T (T),
         To_MaRTE_Param_Ac (AppSched_Param.all'Access),
         Kernel.AppSched_Param_Size_T (AppSched_Param_Size));
   end Pthread_Setappschedparam;

   --------------------------------
   --  Pthread_Getappschedparam  --
   --------------------------------
   function Pthread_Getappschedparam
     (T                   : in     pthread_t;
      AppSched_Param      : in     Scheduling_Parameters_Ac;
      AppSched_Param_Size : access Scheduling_Parameters_Size) return int is
   begin
      return TO_APPSCHD.Pthread_Getappschedparam
        (To_MaRTE_Pthread_T (T),
         To_MaRTE_Param_Ac (AppSched_Param),
         AppSched_Param_Size);
   end Pthread_Getappschedparam;

end System.OS_Interface.Application_Scheduling;
