--  Test for: x86/GPL2007 linux/GPL2007 linux_lib/GPL2007
------------------------------------------------------------------------------
--
--  Counts the number of context switches between round robin tasks
--  and compares that number with the expected value
--  (text_duration/quantum)
--
------------------------------------------------------------------------------

pragma Task_Dispatching_Policy (Round_Robin_Within_Priorities);
with MaRTE_OS;

with System;
with Ada.Real_Time;
with Ada.Dispatching.Round_Robin;
with Text_IO; use Text_IO;

with Reports;

procedure Test_Round_Robin is

   package RT renames Ada.Real_Time;

   Prio : constant System.Any_Priority := 10;

   pragma Priority (Prio);

   Test_Duration : constant Duration := 1.0;

   Last_Task_Id : Natural := 0;
   pragma Volatile (Last_Task_Id);
   CS_Count : Natural := 0;
   pragma Volatile (CS_Count);
   End_Test : Boolean := False;
   pragma Volatile (End_Test);
   Start : Boolean := False;
   pragma Volatile (Start);

   ----------------------
   -- Round_Robin_Task --
   ----------------------
   task type Round_Robin_Task (Prio : System.Priority; Id : Natural) is
      pragma Priority (Prio);
   end Round_Robin_Task;

   task body Round_Robin_Task is
   begin
      Reports.Init;
      delay 0.01;  --  to allow other round-robin tasks to start
      while not Start loop  null; end loop;
      loop
         exit when End_Test;
         if Last_Task_Id /= Id then
            --  context switch detected
            Last_Task_Id := Id;
            CS_Count := CS_Count + 1;
         end if;
         --  Put_Line ("RR task " & Natural'Image (Id));
         --  for I in 1 .. 200_000 loop  null; end loop;
      end loop;
   end Round_Robin_Task;

   T1 : Round_Robin_Task (Prio, 1);
   T2 : Round_Robin_Task (Prio, 2);
   T3 : Round_Robin_Task (Prio, 3);

   Id : constant Natural := 0;

   Quantum : Duration;
   CS_Expected : Natural;

begin
   Reports.Init;
   delay 0.1;  --  to allow other round-robin task to start

     select
        delay Test_Duration;
        End_Test := True;
     then abort
        Start := True;  --  Start test
        loop
           if Last_Task_Id /= Id then
              --  context switch detected
              Last_Task_Id := Id;
              CS_Count := CS_Count + 1;
           end if;
           --  Put_Line ("Main task");
           --  for I in 1 .. 400_000 loop null; end loop;
        end loop;
   end select;

   Quantum :=
     RT.To_Duration (Ada.Dispatching.Round_Robin.Actual_Quantum (Prio));
   CS_Expected := Integer (Test_Duration/Quantum);
   Put_Line ("CS count:" & Natural'Image (CS_Count) & " (expected: " &
             Natural'Image (CS_Expected) & ")");

   --  Allow a 20% error
   Reports.Assert (Float (abs (CS_Expected-CS_Count))
                   / Float (CS_Expected) < 0.20);

   Reports.Test_OK;

end Test_Round_Robin;
