------------------------------------------------------------------------------
--  ------------------         M a R T E   O S         -------------------  --
------------------------------------------------------------------------------
--                                                             V1.7     071213
--
--                  'P r o c e s s o r _ R e g i s t e r s'
--
--                                  Spec
--
--
--  File 'processor_registers.ads'                                    By Mar.
--
--
--  x86 registers.
--
--  ----------------------------------------------------------------------
--   Copyright (C) 2000-2007, Universidad de Cantabria, SPAIN
--
--   MaRTE OS web page: http://marte.unican.es
--   Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
--                      Michael Gonzalez Harbour      mgh@unican.es
--
--  MaRTE OS  is free software; you can  redistribute it and/or  modify it
--  under the terms of the GNU General Public License  as published by the
--  Free Software Foundation;  either  version 2, or (at  your option) any
--  later version.
--
--  MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
--  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
--  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
--  General Public License for more details.
--
--  You should have received  a  copy of  the  GNU General Public  License
--  distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
--  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
--  02111-1307, USA.
--
--  As a  special exception, if you  link this  unit  with other  files to
--  produce an   executable,   this unit  does  not  by  itself cause  the
--  resulting executable to be covered by the  GNU General Public License.
--  This exception does  not however invalidate  any other reasons why the
--  executable file might be covered by the GNU Public License.
--
------------------------------------------------------------------------------

with System; --  for 'Address'
with Basic_Integer_Types; use Basic_Integer_Types;

package Processor_Registers is
   pragma Elaborate_Body;

   -------------------
   -- CPU Registers --
   -------------------
   function Get_ESP_Register return Unsigned_32;
   pragma Import (C, Get_ESP_Register, "get_esp_register");
   --  Defined in 'processor_registers.c'.
   --------------------
   -- Interrupt Flag --
   --------------------
   procedure CLI;
   pragma Inline_Always (CLI);
   pragma Export (C, CLI, "hwi_disable_interrupts");
   procedure STI;
   pragma Inline_Always (STI);

   ---------------------------
   -- Are_Interrupts_Enable --
   ---------------------------
   function Are_Interrupts_Enabled return Boolean;

   --------------------------------------------
   -- Save and restore the "eflags" register --
   --------------------------------------------
   procedure Save_Flags_And_Disable_Interrupts (EFlags : out Integer);
   pragma Inline_Always (Save_Flags_And_Disable_Interrupts);

   function Save_Flags return Integer;
   pragma Inline_Always (Save_Flags);

   procedure Restore_Flags (EFlags : in Integer);
   pragma Inline_Always (Restore_Flags);

   --------------------
   -- Bit Operations --
   --------------------
   procedure Bit_Set (Bit_Field : in out Unsigned_32;
                      Bit       : in Unsigned_32);
   pragma Export (C, Bit_Set, "bit_set");
   pragma Export_Function (Bit_Set, "bit_set",
                             Mechanism => (Bit_Field => Reference,
                                           Bit       => Value));
   pragma Inline_Always (Bit_Set);
   --  Sets to 1 the bit in 'Bit_Field' in the position pointed by 'Bit'
   --  ('Bit' in range 0 .. 31)

   procedure Bit_Reset (Bit_Field : in out Unsigned_32;
                        Bit       : in Unsigned_32);
   pragma Export (C, Bit_Reset, "bit_reset");
   pragma Export_Function (Bit_Reset, "bit_reset",
                             Mechanism => (Bit_Field => Reference,
                                           Bit       => Value));
   pragma Inline_Always (Bit_Reset);
   --  Sets to 0 the bit in 'Bit_Field' in the position pointed by 'Bit'
   --  ('Bit' in range 0 .. 31)

   procedure Bit_Scan_Forward (Bit_Field : in Unsigned_32;
                               Bit       : out Unsigned_32);
   pragma Export (C, Bit_Scan_Forward, "bit_scan_forward");
   pragma Export_Function (Bit_Scan_Forward, "bit_scan_forward",
                             Mechanism => (Bit_Field => Value,
                                           Bit       => Reference));
   pragma Inline_Always (Bit_Scan_Forward);
   --  Returns in 'Bit' the position of the LEST SIGNIFICANT bit set in
   --  'Bit_Field' ('Bit' in range 0 .. 31)

   procedure Bit_Scan_Reverse (Bit_Field : in Unsigned_32;
                               Bit       : out Unsigned_32);
   pragma Export (C, Bit_Scan_Reverse, "bit_scan_reverse");
   pragma Export_Function (Bit_Scan_Reverse, "bit_scan_reverse",
                             Mechanism => (Bit_Field => Value,
                                           Bit       => Reference));
   pragma Inline_Always (Bit_Scan_Reverse);
   --  Returns in 'Bit' the position of the MOST SIGNIFICANT bit set in
   --  'Bit_Field' ('Bit' in range 0 .. 31)

   -------------------
   -- Context Swich --
   -------------------
   procedure Change_Return_Address_Of_Preempted_Task
     (Top_Of_Stack    : in Unsigned_32;
      New_Ret_Address : in System.Address);
   pragma Inline (Change_Return_Address_Of_Preempted_Task);

   procedure Context_Switch (Old_Task : in System.Address;
                             New_Task : in System.Address);
   pragma Inline (Context_Switch);

   procedure Change_To_Context (New_Task : in System.Address);
   pragma Inline (Change_To_Context);

end Processor_Registers;
